<?php

namespace Litecms\Concern\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litepie\Repository\Filter\RequestFilter;
use Litecms\Concern\Forms\Concern as ConcernForm;
use Litecms\Concern\Http\Requests\ConcernRequest;
use Litecms\Concern\Interfaces\ConcernRepositoryInterface;
use Litecms\Concern\Repositories\Eloquent\Filters\ConcernResourceFilter;
use Litecms\Concern\Repositories\Eloquent\Presenters\ConcernListPresenter;

/**
 * Resource controller class for concern.
 */
class ConcernResourceController extends BaseController
{

    /**
     * Initialize concern resource controller.
     *
     *
     * @return null
     */
    public function __construct(ConcernRepositoryInterface $concern)
    {
        parent::__construct();
        $this->form = ConcernForm::setAttributes()->toArray();
        $this->modules = $this->modules(config('litecms.concern.modules'), 'concern', guard_url('concern'));
        $this->repository = $concern;
    }

    /**
     * Display a list of concern.
     *
     * @return Response
     */
    public function index(ConcernRequest $request)
    {

        $pageLimit = $request->input('pageLimit', config('database.pagination.limit'));
        $data = $this->repository
            ->pushFilter(RequestFilter::class)
            ->pushFilter(ConcernResourceFilter::class)
            ->setPresenter(ConcernListPresenter::class)
            ->simplePaginate($pageLimit)
            // ->withQueryString()
            ->toArray();

        extract($data);
        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('concern::concern.names'))
            ->view('concern::concern.index')
            ->data(compact('data', 'meta', 'links', 'modules', 'form'))
            ->output();
    }

    /**
     * Display concern.
     *
     * @param Request $request
     * @param Model   $concern
     *
     * @return Response
     */
    public function show(ConcernRequest $request, ConcernRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('concern::concern.name'))
            ->data(compact('data', 'form', 'modules', 'form'))
            ->view('concern::concern.show')
            ->output();
    }

    /**
     * Show the form for creating a new concern.
     *
     * @param Request $request
     *x
     * @return Response
     */
    public function create(ConcernRequest $request, ConcernRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('concern::concern.name'))
            ->view('concern::concern.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Create new concern.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ConcernRequest $request, ConcernRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $attributes['user_type'] = user_type();
            $repository->create($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.created', ['Module' => trans('concern::concern.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('concern/concern/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/concern/concern'))
                ->redirect();
        }

    }

    /**
     * Show concern for editing.
     *
     * @param Request $request
     * @param Model   $concern
     *
     * @return Response
     */
    public function edit(ConcernRequest $request, ConcernRepositoryInterface $repository)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = $repository->toArray();

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('concern::concern.name'))
            ->view('concern::concern.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();
    }

    /**
     * Update the concern.
     *
     * @param Request $request
     * @param Model   $concern
     *
     * @return Response
     */
    public function update(ConcernRequest $request, ConcernRepositoryInterface $repository)
    {
        try {
            $attributes = $request->all();
            $repository->update($attributes);
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('concern::concern.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('concern/concern/' . $data['id']))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('concern/concern/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the concern.
     *
     * @param Model   $concern
     *
     * @return Response
     */
    public function destroy(ConcernRequest $request, ConcernRepositoryInterface $repository)
    {
        try {
            $repository->delete();
            $data = $repository->toArray();

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('concern::concern.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('concern/concern/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('concern/concern/' .  $repository->getRouteKey()))
                ->redirect();
        }

    }
}
